/*
 * Created on 10.11.2004
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2509 $
 *  $Date: 2006-10-06 12:01:50 +0200 (Fr, 06 Okt 2006) $
 *  $Id: PirolCsvDatasetConnection.java 2509 2006-10-06 10:01:50Z LBST-PF-3\orahn $s
 */
package de.fhOsnabrueck.jump.pirol.datasources.pirolCSV;

import java.awt.Frame;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Map;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jump.feature.AttributeType;
import com.vividsolutions.jump.feature.Feature;
import com.vividsolutions.jump.feature.FeatureCollection;
import com.vividsolutions.jump.feature.FeatureSchema;
import com.vividsolutions.jump.io.datasource.Connection;
import com.vividsolutions.jump.io.datasource.DataSource;
import com.vividsolutions.jump.task.TaskMonitor;

import de.fhOsnabrueck.jump.pirol.utilities.FeatureCollection.PirolFeatureCollection;
import de.fhOsnabrueck.jump.pirol.utilities.apiTools.FeatureCollectionTools;
import de.fhOsnabrueck.jump.pirol.utilities.attributes.AttributeInfo;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.metaData.Attribute2UnitMap;
import de.fhOsnabrueck.jump.pirol.utilities.metaData.MetaInformationHandler;

/**
 * This class provides methods to read and save Datasets from/to CSV-files with PIROL header lines.
 * 
 * @author Ole Rahn, Carsten Schulze
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2509 $
 */
public class PirolCsvDatasetConnection implements Connection {

	protected PirolCSV_Parser csvParser = null;
	protected PirolCsvDatasetDataSource csvDataSource = null;
	protected String fileName = null;
	protected PirolFeatureCollection fc = null;
	
	private boolean readyForReading = false, readyForWriting = false;
	private Frame owner=null;
	
    protected PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);

	// <from Save Data source>
	private static final String X_COORDINATE = new String("x");
	private static final String Y_COORDINATE = new String("y");
	private static final String HEADER_SYMBOL = new String("$");
	private File csvFile = null;
	private FileWriter fout = null;
	// </from Save Data source>
	
	/**
	 * The construktor.
	 * @param csvDataSource the DataSource to read from. The DataSource is important to 
	 * get the specified path and name of the file to save.
	 * @param owner the parent Frame, to handle user dialogs and progess information
	 */
	public PirolCsvDatasetConnection(PirolCsvDatasetDataSource csvDataSource, Frame owner) {
		super();
		this.owner = owner;
		this.csvDataSource = csvDataSource;
	}
	
	
	
    public boolean isError() {
        if (this.fc==null){
		    try {
                this.fc = this.csvParser.createFeatureCollection();
            } catch (Exception e) {
                logger.printError(e.getMessage());
                return true;
            }
		}
        return fc.isEmpty();
    }
    public StringBuffer getErrorMessages() {
        StringBuffer strb = new StringBuffer();
        if (this.fc==null){
		    try {
                this.fc = this.csvParser.createFeatureCollection();
            } catch (Exception e) {
                strb.append(e.getMessage());
                logger.printError(e.getMessage());
            }
		}
        return strb;
    }
    
	protected void openForReading() throws Exception {
		Map dataSourceProps = null;
		if ( this.csvDataSource != null ){
			dataSourceProps = this.csvDataSource.getProperties();
		} else {
			throw new NullPointerException("Member \"csvDataSource\" may not be null.");
		}
		
		if ( dataSourceProps == null ){
			throw new NullPointerException("No dataSource properties were provided.");
		}
		
		if ( dataSourceProps.containsKey(PirolCsvDatasetDataSource.FILENAME_PROP_KEY) ){
		    if (this.readyForWriting)
		        this.closeForWriting();
		    
			fileName = (String)dataSourceProps.get(PirolCsvDatasetDataSource.FILENAME_PROP_KEY);
			
			FileInputStream finStream = new FileInputStream(fileName);
			
			InputStreamReader isReader = new InputStreamReader( finStream );
			
			this.csvParser = new PirolCSV_Parser(new BufferedReader( isReader ));
			
			if (!this.csvParser.isParsed()){
			    int xCol=-1, yCol=-1;
			    String separator = "";
			    
			    xCol = this.csvDataSource.getXColumn();
			    yCol = this.csvDataSource.getYColumn();
			    separator = this.csvDataSource.getSeparator();
			    
			    if ( yCol <= -1 || xCol <= -1 || separator.length()==0 ){
					PirolCsvReaderDialog dialog = new PirolCsvReaderDialog(this.owner,this.csvParser.getHeaderLinesForDialog());
					
					if (!dialog.isFocused()){
						dialog.toFront();
					}
					
					dialog.setVisible(true);
					
					while (dialog.isVisible()){
						//System.out.println("sleeping");
						Thread.sleep(500);
					}

					if (dialog.isInputOK()){
					    xCol = dialog.getXcol();
					    this.csvDataSource.setXColumn(xCol);
					    yCol = dialog.getYcol();
					    this.csvDataSource.setYColumn(yCol);
					    separator = dialog.getSeparator();
					    this.csvDataSource.setSeparator(separator);
					}
			    }
				
				if ( yCol > -1 && xCol > -1 && separator.length()!=0){
					this.csvParser.setXColumn(xCol);
					this.csvParser.setYColumn(yCol);
					
					this.csvParser.setSeparator(separator);
					
					this.readyForReading = true;
				} else {
					//throw new IllegalArgumentException("Abbruch durch User!");
				    this.readyForReading = false;
				    return;
				}
			}
			
		} else {
			this.readyForReading = false;
			throw new NoSuchFieldException("No Property found with name: "+PirolCsvDatasetDataSource.FILENAME_PROP_KEY);
		}
	}
	
	/**
	 * This method opens the {@link java.io.FileWriter}.
	 * @throws IOException if the file exists but is a directory rather than a 
	 * regular file, does not exist but cannot be created, or cannot be opened 
	 * for any other reason
	 */
	private void openForWriting() throws IOException{
		if(this.csvDataSource != null){
			if(this.csvDataSource.getProperties().containsKey(DataSource.FILE_KEY)){
			    if (this.readyForReading)
			        this.closeForReading();
			    
				String path = (String)this.csvDataSource.getProperties().get(DataSource.FILE_KEY);
				if((path.endsWith(".csv")==false)&&((path.endsWith(".txt")==false))){
					path = new String(path + ".csv");
				}
				csvFile = new File(path);
				fout = new FileWriter(csvFile);
				
				this.readyForWriting = true;
			}
		}
	}

	protected FeatureCollection executeQuery( String query ){
		if (this.fc==null){
            if ( this.readyForReading == false ){
                try {
                    this.openForReading();
                } catch (Exception e) {
                    this.logger.printError("Can not open for reading: " + e.getMessage());
                }
            }
		    try {
                this.fc = MetaInformationHandler.createPirolFeatureCollection(this.csvParser.createFeatureCollection());
            } catch (Exception e1) {
                this.logger.printError(e1.getMessage());
                return null;
            }
            try {
                AttributeInfo[] attrInfos = this.csvParser.getColumns();
                Attribute2UnitMap a2um = new Attribute2UnitMap();
                a2um.setUseHTMLLineBreaks(true);
                
                for (int i=0; i<attrInfos.length; i++){
                    a2um.put(attrInfos[i].getUniqueAttributeName(), attrInfos[i].getUnitIdentifier());
                }
                MetaInformationHandler metaInf = new MetaInformationHandler((PirolFeatureCollection)this.fc);
                metaInf.putAttribute2UnitMap(a2um);
                
            } catch (Exception e) {
                this.logger.printMinorError("Can not save unit information: " + e.getMessage());
            } 
		}
		return this.fc;
	}
	
	public FeatureCollection executeQuery(String query, Collection exceptions,
			TaskMonitor monitor) {
        this.logger.printDebug("executeQuery: >"+query+"<");
		if ( this.readyForReading == false ){
			try {
				this.openForReading();
			} catch (Exception e) {
				exceptions.add(e);
				e.printStackTrace();
				return null;
			}
		}
        if (this.fc==null){
            this.executeQuery(query);
        }
		
		return this.fc;
	}


	public FeatureCollection executeQuery(String query, TaskMonitor monitor)
			throws Exception {
        this.logger.printDebug("executeQuery_2: >"+query+"<");
		if ( this.readyForReading == false ){
			this.openForReading();
		}
		return executeQuery(query);
	}


	/**
	 * This method is called to write data into the file.
	 * @param query ignored
	 * @param fc the data to save
	 * @param monitor something
	 * @throws java.lang.Exception maybe
	 * @see com.vividsolutions.jump.io.datasource.Connection#executeUpdate(java.lang.String, com.vividsolutions.jump.feature.FeatureCollection, com.vividsolutions.jump.task.TaskMonitor)
	 */
	public void executeUpdate(String query, FeatureCollection fc, TaskMonitor monitor) throws Exception {
	    this.logger.printDebug("<" + this.csvDataSource.getSeparator() + ">");

		if(fc != null){
			//opens the connection. #close is called by JUMP.
			//this.open();

		    this.openForWriting();
			
			//get the header
			FeatureSchema schema = fc.getFeatureSchema();
			int geoIndex = schema.getGeometryIndex();
			String names = new String(HEADER_SYMBOL);
			String units = new String(HEADER_SYMBOL);
			String attributes = new String(HEADER_SYMBOL);
			
			String separator = this.csvDataSource.getSeparator();
            
            Attribute2UnitMap a2um = null;
            
            if (PirolFeatureCollection.class.isInstance(fc)) {
                MetaInformationHandler metaInfHandler = new MetaInformationHandler((PirolFeatureCollection)fc);
                a2um = metaInfHandler.getAttribute2UnitMap();
            }
			
            if (a2um==null) {
                // TODO: looks like this can never work, since a new DataSource is created to save the layer...
                this.logger.printDebug("a2um==" + a2um);
                //this.logger.printDebug(metaInfHandler.)
            }
            
			if (separator.length()==0)
			    separator = "\t";
            
			for(int i=0; i<schema.getAttributeCount(); i++){
				if(i != geoIndex){
					//build the name-String
					String n = schema.getAttributeName(i);
					names = new String(names + n + separator);
					
					//the units ara not saved yet, so dummies are used.
                    if (a2um==null) {
                        units = new String(units + "k.a." + separator);
                    } else {
                        this.logger.printDebug("a2um conatins (" + n + ") " + a2um.containsAttribute(n));
                        units = new String(units + (a2um.containsAttribute(n)?a2um.getUnitString(n):"k.a.") + separator);
                    }
					
					//build the attributes-String
					AttributeType aType = schema.getAttributeType(i);
					attributes = new String(attributes + aType.toString().toLowerCase() + separator);
				}
				else{
					String type = AttributeType.DOUBLE.toString().toLowerCase();
					names = new String(names + X_COORDINATE + separator + Y_COORDINATE + separator);
                    if (a2um==null) {
                        units = new String(units + "k.a." + separator + "k.a." + separator);
                    } else {
                        // TODO: think about, how to get the correct unit string in here!
                        units = new String(units + "k.a." + separator + "k.a." + separator);
                    }
					attributes = new String(attributes + type + separator + type + separator);
				}
			}
			//write the header
			fout.write(names.trim() + "\n");
			fout.write(units.trim() + "\n");
			fout.write(attributes.trim() + "\n");
			
			
			//get the data
			Feature[] features = FeatureCollectionTools.FeatureCollection2FeatureArray(fc);

			for (int i=0; i<features.length; i++){
				Feature feat = features[i];
				Object[] o = feat.getAttributes();
				
				for(int j = 0; j < o.length; j++){
					String s = null;
					
					//convert the geometry information
					if((o[j] instanceof Geometry)){
						Geometry g = (Geometry)o[j];
						//the centroid should be the better way for various geometry
						Point p = g.getCentroid();
						s = new String(p.getX() + separator + p.getY());
					}
					else{
						s = o[j].toString();
					}
					
					fout.write(s + separator);
				}
				//write the data
				fout.write("\n");
			}
		}
	}

	
	protected void closeForReading(){
	    try {
			if (this.readyForReading){
				this.csvParser.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.readyForReading = false;
	}
	protected void closeForWriting(){
	    try {
		    if (this.fout != null)
		        fout.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.readyForWriting = false;
	}

	public void close() {
	    // close reading part
		this.closeForReading();
	    // close writing part
		this.closeForWriting();
	}
	

}
