/*
 * Created on 22.06.2005 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2509 $
 *  $Date: 2006-10-06 12:01:50 +0200 (Fr, 06 Okt 2006) $
 *  $Id: SaveImageAsAction.java 2509 2006-10-06 10:01:50Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.actions;

import java.awt.AWTException;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.sun.media.jai.codec.PNGEncodeParam;
import com.sun.media.jai.codecimpl.PNGCodec;
import com.sun.media.jai.codecimpl.PNGImageEncoder;

import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;

/**
 * Generic action (to be added to GUI controlls such as buttons) to save the contents of an AWT component to an image file.
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2509 $
 * 
 */
public class SaveImageAsAction extends AbstractAction {

    private static final long serialVersionUID = -3490102728600683489L;
    protected Component component2Draw = null;
    protected Dialog parentDialog;
    protected PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);
    

    public SaveImageAsAction(Dialog parentDialog, Component component2Draw) {
        super(PirolPlugInMessages.getString("save-image"));
        this.component2Draw = component2Draw;
        this.parentDialog = parentDialog;
        
        this.putValue(AbstractAction.NAME, PirolPlugInMessages.getString("save-image"));
        this.putValue(AbstractAction.SHORT_DESCRIPTION, PirolPlugInMessages.getString("save-image-tooltip"));
    }

    /**
     *@param titleText
     */
    public SaveImageAsAction(Dialog parentDialog, Component component2Draw, String titleText) {
        super(titleText);
        this.component2Draw = component2Draw;
        this.parentDialog = parentDialog;
        
        this.putValue(AbstractAction.NAME, titleText);
        this.putValue(AbstractAction.SHORT_DESCRIPTION, PirolPlugInMessages.getString("save-image-tooltip"));
    }

    protected BufferedImage getScreenImage() throws AWTException{
        Robot robot = new Robot();
        Point location = this.component2Draw.getLocationOnScreen();
        
        Rectangle visibleRect = new Rectangle(location, this.component2Draw.getSize());
        
        return robot.createScreenCapture(visibleRect);
    }
    
    /**
     * Defines what is supposed to happen if the control using this action is activated (e.g. clicked if it's a button)
     *@param actionEvent
     */
    public void actionPerformed(ActionEvent actionEvent) {
        BufferedImage bim = null;
        
        try {
            if (!javax.media.j3d.Canvas3D.class.isInstance(this.component2Draw)){
                bim = new BufferedImage(this.component2Draw.getWidth(), this.component2Draw.getHeight(), BufferedImage.TYPE_INT_RGB);
                Graphics2D grph = bim.createGraphics();
                this.component2Draw.paint(grph);
                grph.dispose();
            } else {
                try {
                    bim = this.getScreenImage();
                } catch (AWTException e1) {
                    this.logger.printError(e1.getMessage());
                }
            }
        } catch (Throwable e) {
            // javax.media.j3d.Canvas3D not found -> no Java3D -> it is a 2D component
            try {
                bim = this.getScreenImage();
            } catch (AWTException e1) {
                this.logger.printError(e1.getMessage());
            }
        }
        
        JFileChooser chooser = new JFileChooser();
        chooser.setFileFilter(
                new FileFilter(){
                    public boolean accept(File f) {
					    if (f.isDirectory()) {
					        return true;
					    }
					
					    String extension = "";
					    if (f.getName().lastIndexOf(".")>-1) {
					        extension = f.getName().substring(f.getName().lastIndexOf("."));
						    if (extension != null) {
								if (extension.toLowerCase().equals(".png")) {
								        return true;
								}
							    return false;
						    }
					    } 
					    return false;
                    }

                    public String getDescription() {
                        return "*.png";
                    }
                });
        int result = chooser.showSaveDialog(this.parentDialog);
        
        if (result != JFileChooser.APPROVE_OPTION) return;
        
        String fileName = chooser.getSelectedFile().getAbsolutePath();
        
        if (!fileName.toLowerCase().endsWith(".png")){
            fileName += ".png";
        }
        
        try {
            FileOutputStream imgOut = new FileOutputStream(fileName);
            
            PNGEncodeParam param = new PNGEncodeParam.RGB();

            PNGImageEncoder encoder = (PNGImageEncoder) PNGCodec.createImageEncoder("png", imgOut, param);
            encoder.encode(bim);
            imgOut.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        
    }

 
}
