/*
 * Created on 23.05.2005 for PIROL
 * 
 * SVN header information:
 * $Author: LBST-PF-3\orahn $
 * $Rev: 2446 $
 * $Date: 2006-09-12 14:57:25 +0200 (Di, 12 Sep 2006) $
 * $Id: IntegerInputDocument.java 2446 2006-09-12 12:57:25Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.documents;

import javax.swing.JDialog;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * A plain document for Integer values only. The default element structure for 
 * this document is a map of the lines in the text. The Element returned by 
 * getDefaultRootElement is a map of the lines, and each child element 
 * represents a line. This model does not maintain any character level 
 * attributes, but each line can be tagged with an arbitrary set of attributes. 
 * Line to offset, and offset to line translations can be quickly performed 
 * using the default root element. The structure information of the 
 * DocumentEvent's fired by edits will indicate the line structure changes. 
 * 
 * 
 * The default content storage management is performed by a gapped buffer 
 * implementation (GapContent). It supports editing reasonably large documents 
 * with good efficiency when the edits are contiguous or clustered, as is 
 * typical. 
 * @author Carsten Schulze
 * @author <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck
 * <br>Project: PIROL (2005)
 * <br>Subproject: Daten- und Wissensmanagement
 */
public class IntegerInputDocument extends PlainDocument {

    private static final long serialVersionUID = 3496661401696742340L;
    
    /**just to "beep"!*/
	private static JDialog dialog = new JDialog();
	/**
	 * Inserts some content into the document. Inserting content causes a write 
	 * lock to be held while the actual changes are taking place, followed by 
	 * notification to the observers on the thread that grabbed the write lock.
	 * @param offset the starting offset >= 0
	 * @param string the string to insert; does nothing with null/empty strings
	 * @param attributeSet the attributes for the inserted content
	 * @throws BadLocationException if the given insert position is not a valid 
	 * position within the document
	 */
	public void insertString(int offset, String string, AttributeSet attributeSet) throws BadLocationException{
		try{
			Integer.parseInt(string);
			super.insertString(offset,string,attributeSet);
		}
		catch(NumberFormatException e){
			dialog.getToolkit().beep();
		}
	}
}
