/*
 * Created on 13.09.2006 for PIROL
 * 
 * SVN header information:
 *  $Rev$
 *  $Author$
 *  $Date$
 *  $Id$
 */
/**
 * 
 */
package de.fhOsnabrueck.jump.pirol.ui.panels;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.border.EtchedBorder;

import com.vividsolutions.jts.geom.Coordinate;

import de.fhOsnabrueck.jump.pirol.ui.eventHandling.LoadPropertiesEvent;
import de.fhOsnabrueck.jump.pirol.ui.eventHandling.LoadPropertiesListener;
import de.fhOsnabrueck.jump.pirol.ui.tools.ValueChecker;
import de.fhOsnabrueck.jump.pirol.utilities.Properties.PropertiesHandler;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;
import de.fhOsnabrueck.jump.pirol.utilities.projection.LayerProjector;
import de.fhOsnabrueck.jump.pirol.utilities.projection.ReprojectionDescription;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.AbstractProjection;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.GaussKruegerProjection;
import de.fhOsnabrueck.jump.pirol.utilities.projection.Projections.Wgs84Projection;
import de.fhOsnabrueck.jump.pirol.utilities.settings.PirolPlugInSettings;

/**
 * TODO: comment class
 *
 * <br><br><b>Last change by $Author$ on $Date$</b>
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2006),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev$
 * 
 */
public class ChangeProjectionPanel extends Box implements LoadPropertiesListener, ValueChecker, ActionListener {

	private static final long serialVersionUID = -2440098484330998901L;

	protected JComboBox inProjBox = null, outProjBox = null;
    
    protected SpinnerPanel zonePanel = new SpinnerPanel(3, 0, 120, 1, PirolPlugInMessages.getString("target-meridian") + ":");
    
    public static final Dimension comboBoxDimension = new Dimension(150, 25), preferredDimension = new Dimension(500, 30);

    protected static final String projectionDirStr = "helmert-parameter";
    public static File projectionDir = null;
    
    protected ChooseAndLoadPropertyFilesFromDirectoryBox storedHelmertParameters = null;
    protected PropertiesHandler propsHandler = null;

    protected JLabel currLoaded = new JLabel(""), sourceProjectionLabel = null, targetProjectionLabel = null;

	public ChangeProjectionPanel() {
		super(BoxLayout.Y_AXIS);
		
        if (projectionDir==null){
        	projectionDir = new File(PirolPlugInSettings.configDirectory().getAbsolutePath() + File.separator + projectionDirStr + File.separator);
        }
        
        this.initialize();
	}
	
    protected JComboBox getInputProjectionsBox(){
        if (this.inProjBox==null){
            this.inProjBox = new JComboBox(LayerProjector.validInputProjections);
            this.inProjBox.setMaximumSize(comboBoxDimension);
            this.inProjBox.setPreferredSize(comboBoxDimension);
            
            this.inProjBox.addActionListener(this);
        }
        return this.inProjBox;
    }
    
    protected JComboBox getOutputProjectionsBox(){
        if (this.outProjBox==null){
            this.outProjBox = new JComboBox(LayerProjector.validOutputProjections);
            this.outProjBox.setMaximumSize(comboBoxDimension);
            this.outProjBox.setPreferredSize(comboBoxDimension);
            
            this.outProjBox.addActionListener(this);
        }
        return this.outProjBox;
    }
    
    /**
     * 
     *@return the choosen Gauss-Krueger zone (3 degree)
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public int getGkZone() {
        return zonePanel.getIntValue();
    }
    

    
    /**
     *sets the Gauss-Krueger zone (3 degree) and disables the spinner panel
     *@param zone the desired zone
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public void setGkZone(int zone) {
        this.zonePanel.setIntValue(zone);
        this.zonePanel.setEnabled(false);
    }
    
    /**
     * 
     *@return the choosen target projection as a string
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public String getOutputProjection() {
        return outProjBox.getSelectedItem().toString();
    }

    /**
     * 
     *@return the choosen source projection as a string
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public String getInputProjection() {
        return inProjBox.getSelectedItem().toString();
    }
    
    
    /**
     * Sets the target projection to the given value and disables user interaction
     * on this combo box
     *@param op the desired projection as String (nothing will happen, if the projection doesn't match a known one)
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public void setOutputProjection(String op) {
        for (int i=0; i< this.getOutputProjectionsBox().getItemCount(); i++){
            if (this.getOutputProjectionsBox().getItemAt(i).toString().equals(op)){
                this.getOutputProjectionsBox().setSelectedIndex(i);
                this.getOutputProjectionsBox().setEditable(false);
                this.getOutputProjectionsBox().setEnabled(false);
                return;
            }
        }
        
    }

    /**
     * Sets the source projection to the given value and disables user interaction
     * on this combo box
     *@param ip the current projection as String (nothing will happen, if the projection doesn't match a known one)
     *
     *@see AbstractProjection#KEY_GAUSS_KRUEGER
     *@see AbstractProjection#KEY_WGS84
     */
    public void setInputProjection(String ip) {
        for (int i=0; i< this.getInputProjectionsBox().getItemCount(); i++){
            if (this.getInputProjectionsBox().getItemAt(i).toString().equals(ip)){
                this.getInputProjectionsBox().setSelectedIndex(i);
                this.getInputProjectionsBox().setEditable(false);
                this.getInputProjectionsBox().setEnabled(false);
                return;
            }
        }
        
    }
    
    

    public PropertiesHandler getHelmertParameters() {
        return propsHandler;
    }

    public void setPropsHandler(PropertiesHandler propsHandler) {
        this.propsHandler = propsHandler;
    }

    /**
     *@inheritDoc
     */
    public void propertiesLoaded(LoadPropertiesEvent lpe) {
        this.propsHandler = lpe.getPropertiesHandler();
        
        String file = "-";
        
        if (this.propsHandler!=null){
            file = this.propsHandler.getPropertiesFile();
            
            if (file.indexOf(File.separator)>-1){
                file = file.substring(file.lastIndexOf(File.separator)+1);
            }
            
            if (file.indexOf(".")>-1){
                file = file.substring(0,file.lastIndexOf("."));
            }
            
        } 
        
        this.currLoaded.setText(file);
    }

    /**
     * This method initializes this
     */
    protected void initialize() {
        this.setSize(preferredDimension.width, 400);
        
        
        this.add(Box.createVerticalStrut(10));
        
        Box row = Box.createHorizontalBox();
        row.add(Box.createHorizontalStrut(10));
        sourceProjectionLabel = new JLabel(PirolPlugInMessages.getString("source-projection")); 
        row.add(this.sourceProjectionLabel);
        row.add(Box.createGlue());
        row.add(this.getInputProjectionsBox());
        row.add(Box.createHorizontalStrut(10));
        row.setMaximumSize(ChangeProjectionPanel.preferredDimension);
        this.add(row);
        
        this.add(Box.createVerticalStrut(20));
        
        row = Box.createHorizontalBox();
        row.add(Box.createHorizontalStrut(10));
        targetProjectionLabel = new JLabel(PirolPlugInMessages.getString("target-projection")); 
        row.add(targetProjectionLabel);
        row.add(Box.createGlue());
        row.add(this.getOutputProjectionsBox());
        row.add(Box.createHorizontalStrut(10));
        row.setMaximumSize(ChangeProjectionPanel.preferredDimension);
        this.add(row);
        
        this.add(Box.createVerticalStrut(20));
        
        row = Box.createHorizontalBox();
        this.zonePanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        row.add(this.zonePanel);
        row.setMaximumSize(ChangeProjectionPanel.preferredDimension);
        this.add(row);
        
        this.add(Box.createVerticalStrut(20));
        
        this.storedHelmertParameters = new ChooseAndLoadPropertyFilesFromDirectoryBox("load-helmert-parameters",projectionDir);
        this.storedHelmertParameters.addLoadPropertiesListener(this);
        row = Box.createHorizontalBox();
        row.add(this.storedHelmertParameters);
        row.setMaximumSize(ChangeProjectionPanel.preferredDimension);
        this.add(row);
        
        this.add(Box.createVerticalStrut(5));
        //content.add(Box.createVerticalStrut(20));
        // TODO
        row = Box.createHorizontalBox();
        row.add(Box.createHorizontalStrut(10));
        row.add(new JLabel(PirolPlugInMessages.getString("currently-loaded-helmert-parameters") + "  "));
        row.add(Box.createGlue());
        row.add(this.currLoaded);
        row.add(Box.createHorizontalStrut(10));
        row.setMaximumSize(ChangeProjectionPanel.preferredDimension);
        this.add(row);
        
        this.add(Box.createGlue());
        this.add(Box.createVerticalStrut(30));
        
    }


    
    public ReprojectionDescription getReprojectionDescription(Coordinate exampleCoord){
        AbstractProjection startProj = null, targetProj = null;
        
        if (this.getInputProjection().equals(AbstractProjection.KEY_WGS84)){
            startProj = new Wgs84Projection();
        } else if (this.getInputProjection().equals(AbstractProjection.KEY_GAUSS_KRUEGER)){
            int zone = GaussKruegerProjection.guessGkZone(exampleCoord);
            
            startProj = new GaussKruegerProjection(zone);
            ((GaussKruegerProjection)startProj).fillInHelmertParametersFromProperties(this.getHelmertParameters());
        } else {
            throw new IllegalArgumentException("given input projection is not supported, yet: " + this.getInputProjection());
        }
        
        if (this.getOutputProjection().equals(AbstractProjection.KEY_WGS84)){
            targetProj = new Wgs84Projection();
        } else if (this.getOutputProjection().equals(AbstractProjection.KEY_GAUSS_KRUEGER)){
            int zone = this.getGkZone();
            
            targetProj = new GaussKruegerProjection(zone);
            ((GaussKruegerProjection)targetProj).fillInHelmertParametersFromProperties(this.getHelmertParameters());
        } else {
            throw new IllegalArgumentException("given target projection is not supported, yet" + this.getOutputProjection());
        }
        
        return new ReprojectionDescription(startProj, targetProj);
    }

    /**
     * Handle changes in the projection drop downs
     *@inheritDoc
     */
    public void actionPerformed(ActionEvent arg0) {
        if (arg0.getSource().equals(this.outProjBox)){
            if (this.outProjBox.getSelectedItem().equals(AbstractProjection.KEY_GAUSS_KRUEGER)){
                this.zonePanel.setEnabled(true);
            } else {
                this.zonePanel.setEnabled(false);
            }
        }
    }

    /**
     *@inheritDoc
     */
    public boolean areValuesOk() {
        return true;
    }

	public void setSourceProjectionLabelText(String text) {
		sourceProjectionLabel.setText(text);
	}
    
	public void setTargetProjectionLabelText(String text) {
		targetProjectionLabel.setText(text);
	}
}
